import os

import pandas as pd
from numpy import mean
import pingouin as pg

from config import Config


def task_aggregated():
    acc_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_ratings.csv",
        delimiter=","
    )
    tct_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_times.csv",
        delimiter=","
    )
    ueq_df = pd.read_csv(
        f"{Config.DataOutputPath}/ueq.csv",
        delimiter=","
    )

    acc_grp = acc_df.groupby(
        by=["PID", "Survey ID", "VL-Bar", "VL-Scatter", "ChartType", "Adaptation"]
    ).mean().reset_index()
    tct_grp = tct_df.groupby(
        by=["PID", "Survey ID", "VL-Bar", "VL-Scatter", "ChartType", "Adaptation"]
    ).mean().reset_index()

    out_df = pd.DataFrame()
    for key in ["PID", "VL-Bar", "VL-Scatter", "ChartType", "Adaptation"]:
        out_df[key] = acc_grp[key]

    out_df["VL-Bar"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Bar"] if row["ChartType"] == "Bar" else None
    )
    out_df["VL-Scatter"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Scatter"] if row["ChartType"] == "Scatter" else None
    )

    out_df["Time Bar"] = tct_grp.apply(axis=1,
        func=lambda row: row["Time"] if row["ChartType"] == "Bar" else None
    )
    out_df["Time Scatter"] = tct_grp.apply(axis=1,
        func=lambda row: row["Time"] if row["ChartType"] == "Scatter" else None
    )

    out_df["Accuracy Bar"] = acc_grp.apply(axis=1,
        func=lambda row: row["Answer"] if row["ChartType"] == "Bar" else None
    )
    out_df["Accuracy Scatter"] = acc_grp.apply(axis=1,
        func=lambda row: row["Answer"] if row["ChartType"] == "Scatter" else None
    )

    for ux_name in ["Dependability", "Usefulness", "Intuitive Use"]:
        out_df[f"{ux_name} Bar"] = out_df.apply(axis=1,
            func=lambda row: mean(
                ueq_df[(ueq_df["PID"] == row["PID"]) & (ueq_df["ChartType"] == "Bar") & (ueq_df["Adaptation"] == row["Adaptation"])
                       & (ueq_df["QuestionGroup"] == ux_name)
                       ]["Rating"]
            ) if row["ChartType"] == "Bar" else None
        )
        out_df[f"{ux_name} Scatter"] = out_df.apply(axis=1,
            func=lambda row: mean(
                ueq_df[(ueq_df["PID"] == row["PID"]) & (ueq_df["ChartType"] == "Scatter") & (ueq_df["Adaptation"] == row["Adaptation"])
                       & (ueq_df["QuestionGroup"] == ux_name)
                       ]["Rating"]
            ) if row["ChartType"] == "Scatter" else None
        )

    # out_df = out_df.sample(frac=1)
    out_df.to_csv(f"{Config.JaspOutputPath}/aggregated_tasks.csv", sep=";")
    return out_df


def complete_vis_aggregated():
    acc_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_ratings.csv",
        delimiter=","
    )
    tct_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_times.csv",
        delimiter=","
    )
    ueq_df = pd.read_csv(
        f"{Config.DataOutputPath}/ueq.csv",
        delimiter=","
    )
    fam_df = pd.read_csv(
        f"{Config.DataOutputPath}/familiarity.csv",
        delimiter=","
    )
    pref_df = pd.read_csv(
        f"{Config.DataOutputPath}/preference.csv",
        delimiter=","
    )

    acc_grp_bar = acc_df[acc_df["ChartType"] == "Bar"].groupby(
        by=["PID", "Survey ID", "VL-Bar", "VL-Scatter"]
    ).mean().reset_index()
    acc_grp_scatter = acc_df[acc_df["ChartType"] == "Scatter"].groupby(
        by=["PID", "Survey ID"]
    ).mean().reset_index()
    tct_grp_bar = tct_df[tct_df["ChartType"] == "Bar"].groupby(
        by=["PID", "Survey ID"]
    ).mean().reset_index()
    tct_grp_scatter = tct_df[tct_df["ChartType"] == "Scatter"].groupby(
        by=["PID", "Survey ID"]
    ).mean().reset_index()
    ueq_grp_bar = ueq_df[ueq_df["ChartType"] == "Bar"].groupby(
        by=["PID", "Survey ID", "QuestionGroup"]
    ).mean().reset_index()
    ueq_grp_scatter = ueq_df[ueq_df["ChartType"] == "Scatter"].groupby(
        by=["PID", "Survey ID", "QuestionGroup"]
    ).mean().reset_index()

    out_df = pd.DataFrame()
    for key in ["PID", "VL-Bar", "VL-Scatter"]:
        out_df[key] = acc_grp_bar[key]

    out_df["Time Bar"] = tct_grp_bar["Time"]
    out_df["Time Scatter"] = tct_grp_scatter["Time"]
    out_df["Accuracy Bar"] = acc_grp_bar["Answer"]
    out_df["Accuracy Scatter"] = acc_grp_scatter["Answer"]

    out_df["Dependability Bar"] = ueq_grp_bar[ueq_grp_bar["QuestionGroup"] == "Dependability"]["Rating"].values
    out_df["Dependability Scatter"] = ueq_grp_scatter[ueq_grp_scatter["QuestionGroup"] == "Dependability"]["Rating"].values
    out_df["Usefulness Bar"] = ueq_grp_bar[ueq_grp_bar["QuestionGroup"] == "Usefulness"]["Rating"].values
    out_df["Usefulness Scatter"] = ueq_grp_scatter[ueq_grp_scatter["QuestionGroup"] == "Usefulness"]["Rating"].values
    out_df["Intuitive Use Bar"] = ueq_grp_bar[ueq_grp_bar["QuestionGroup"] == "Intuitive Use"]["Rating"].values
    out_df["Intuitive Use Scatter"] = ueq_grp_scatter[ueq_grp_scatter["QuestionGroup"] == "Intuitive Use"]["Rating"].values

    for vis_name in ["Pie Chart", "Bar Chart", "Line Chart", "Area Chart", "Scatterplot", "Radar Chart", "Treemap",
                     "Network Graph"]:
        out_df["Fam: " + vis_name] = fam_df[vis_name]

    out_df["Pref Bar Adapt"] = pref_df.apply(axis=1,
        func=lambda row: 1 if row["BarAdapt"] == "A" else 0
    )
    out_df["Pref Scatter Adapt"] = pref_df.apply(axis=1,
        func=lambda row: 1 if row["ScatterAdapt"] == "A" else 0
    )

    out_df.to_csv(f"{Config.JaspOutputPath}/vis_aggregated.csv", sep=";")
    return out_df


def task_single():
    acc_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_ratings.csv",
        delimiter=","
    )
    tct_df = pd.read_csv(
        f"{Config.DataOutputPath}/answer_times.csv",
        delimiter=","
    )

    out_df = pd.DataFrame()
    for key in ["PID", "VL-Bar", "VL-Scatter", "ChartType", "Adaptation", "TaskGroup"]:
        out_df[key] = acc_df[key]

    out_df["VL-Bar"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Bar"] if row["ChartType"] == "Bar" else None
    )
    out_df["VL-Scatter"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Scatter"] if row["ChartType"] == "Scatter" else None
    )
    out_df["VL-Bar Ordinal"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Bar"] * 100 if row["ChartType"] == "Bar" else None
    )
    out_df["VL-Scatter Ordinal"] = out_df.apply(axis=1,
        func=lambda row: row["VL-Scatter"] * 100 if row["ChartType"] == "Scatter" else None
    )

    out_df["Time Bar"] = tct_df.apply(axis=1,
        func=lambda row: row["Time"] if row["ChartType"] == "Bar" else None
    )
    out_df["Time Scatter"] = tct_df.apply(axis=1,
        func=lambda row: row["Time"] if row["ChartType"] == "Scatter" else None
    )

    out_df["Accuracy Bar"] = acc_df.apply(axis=1,
        func=lambda row: row["Answer"] if row["ChartType"] == "Bar" else None
    )
    out_df["Accuracy Scatter"] = acc_df.apply(axis=1,
        func=lambda row: row["Answer"] if row["ChartType"] == "Scatter" else None
    )

    # out_df = out_df.sample(frac=1)
    out_df.to_csv(f"{Config.JaspOutputPath}/single_tasks.csv", sep=";")
    return out_df


if __name__ == '__main__':
    complete_vis_aggregated()
    task_aggregated()
    task_single()
